@@ -25,7 +25,7 @@
  *
  ********************************************************************
 
- last mod: $Id: ao_wav.c 16982 2010-03-18 19:50:04Z xiphmont $
+ last mod: $Id: ao_wav.c 17718 2010-12-06 20:09:29Z xiphmont $
 
  ********************************************************************/
 
@@ -101,7 +101,7 @@ static ao_info ao_wav_info =
 	AO_FMT_LITTLE,
 	0,
 	ao_wav_options,
-        4
+        sizeof(ao_wav_options)/sizeof(*ao_wav_options)
 };
 
 typedef struct ao_wav_internal
@@ -275,6 +275,7 @@ static void ao_wav_device_clear(ao_device *device)
 	ao_wav_internal *internal = (ao_wav_internal *) device->internal;
 
 	free(internal);
+        device->internal=NULL;
 }
 
 
