@@ -23,7 +23,7 @@
  *
  ********************************************************************
 
- last mod: $Id: ao_esd.c 16985 2010-03-19 03:04:44Z xiphmont $
+ last mod: $Id: ao_esd.c 17719 2010-12-06 20:13:20Z xiphmont $
 
  ********************************************************************/
 
@@ -40,7 +40,7 @@
 
 extern char **environ;
 
-static char *ao_esd_options[] = {"host","matrix","verbose","quiet","debug"};
+static char *ao_esd_options[] = {"server","host","matrix","verbose","quiet","debug"};
 static ao_info ao_esd_info =
 {
 	AO_TYPE_LIVE,
@@ -51,7 +51,7 @@ static ao_info ao_esd_info =
 	AO_FMT_NATIVE,
 	40,
 	ao_esd_options,
-	5
+        sizeof(ao_esd_options)/sizeof(*ao_esd_options)
 };
 
 
@@ -146,7 +146,7 @@ int ao_plugin_set_option(ao_device *device, const char *key, const char *value)
 {
 	ao_esd_internal *internal = (ao_esd_internal *) device->internal;
 
-	if (!strcmp(key, "host")) {
+	if (!strcmp(key, "host") || !strcmp(key, "server")) {
 		if(internal->host) free(internal->host);
 		internal->host = strdup(value);
 	}
@@ -303,4 +303,5 @@ void ao_plugin_device_clear(ao_device *device)
 
   if(internal->host) free(internal->host);
   free(internal);
+  device->internal=NULL;
 }
