@@ -25,7 +25,7 @@
  *
  ********************************************************************
 
- last mod: $Id: ao_macosx.c 17011 2010-03-24 06:48:31Z xiphmont $
+ last mod: $Id: ao_macosx.c 17718 2010-12-06 20:09:29Z xiphmont $
 
  ********************************************************************/
 /* The MacOS X CoreAudio framework doesn't mesh as simply as some
@@ -33,6 +33,8 @@
    audio samples rather than having them pushed at it (which is nice
    when you are wanting to do good buffering of audio).  */
 
+#include <CoreAudio/CoreAudio.h>
+#include <CoreServices/CoreServices.h>
 #include <AudioUnit/AudioUnit.h>
 #include <AudioUnit/AUComponent.h>
 #include <stdio.h>
@@ -62,7 +64,7 @@ static ao_info ao_macosx_info =
 	AO_FMT_NATIVE,
 	30,
 	ao_macosx_options,
-	5
+	sizeof(ao_macosx_options)/sizeof(*ao_macosx_options)
 };
 
 static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
@@ -71,7 +73,8 @@ static pthread_cond_t cond = PTHREAD_COND_INITIALIZER;
 typedef struct ao_macosx_internal
 {
   /* Stuff describing the CoreAudio device */
-  AudioComponentInstance       outputAudioUnit;
+  ComponentInstance       outputAudioUnit;
+  int                     output_p;
 
   /* Keep track of whether the output stream has actually been
      started/stopped */
@@ -187,7 +190,7 @@ int ao_plugin_device_init(ao_device *device)
 {
   ao_macosx_internal *internal;
 
-  internal = (ao_macosx_internal *) malloc(sizeof(ao_macosx_internal));
+  internal = (ao_macosx_internal *) calloc(1,sizeof(ao_macosx_internal));
 
   if (internal == NULL)	
     return 0; /* Could not initialize device memory */
@@ -200,6 +203,16 @@ int ao_plugin_device_init(ao_device *device)
   device->internal = internal;
   device->output_matrix_order = AO_OUTPUT_MATRIX_COLLAPSIBLE;
   device->output_matrix = strdup("L,R,C,LFE,BL,BR,CL,CR,BC,SL,SR");
+
+  CFRunLoopRef theRunLoop = NULL;
+  AudioObjectPropertyAddress theAddress = {
+    kAudioHardwarePropertyRunLoop,
+    kAudioObjectPropertyScopeGlobal,
+    kAudioObjectPropertyElementMaster
+  };
+  AudioObjectSetPropertyData(kAudioObjectSystemObject, &theAddress, 0,
+                             NULL, sizeof(CFRunLoopRef), &theRunLoop);
+
   return 1; /* Memory alloc successful */
 }
 
@@ -224,8 +237,8 @@ int ao_plugin_open(ao_device *device, ao_sample_format *format)
 {
   ao_macosx_internal *internal = (ao_macosx_internal *) device->internal;
   OSStatus result = noErr;
-  AudioComponent comp;
-  AudioComponentDescription desc;
+  Component comp;
+  ComponentDescription desc;
   AudioStreamBasicDescription requestedDesc;
   AURenderCallbackStruct      input;
   UInt32 i_param_size;
@@ -237,18 +250,19 @@ int ao_plugin_open(ao_device *device, ao_sample_format *format)
   desc.componentFlags = 0;
   desc.componentFlagsMask = 0;
 
-  comp = AudioComponentFindNext (NULL, &desc);
+  comp = FindNextComponent (NULL, &desc);
   if (comp == NULL) {
     aerror("Failed to start CoreAudio: AudioComponentFindNext returned NULL");
     return 0;
   }
 
   /* Open & initialize the default output audio unit */
-  result = AudioComponentInstanceNew (comp, &internal->outputAudioUnit);
+  result = OpenAComponent (comp, &internal->outputAudioUnit);
   if (result) {
     aerror("AudioComponentInstanceNew() error => %d\n",(int)result);
     return 0;
   }
+  internal->output_p=1;
 
   /* Request desired format of the audio unit.  Let HAL do all
      conversion since it will probably be doing some internal
@@ -475,56 +489,61 @@ int ao_plugin_close(ao_device *device)
   // start now.
 
   pthread_mutex_lock(&mutex);
-  internal->isStopping = true;
 
-  if(!internal->started && internal->validByteCount){
-    status = AudioOutputUnitStart(internal->outputAudioUnit);
-    adebug("Starting audio output unit\n");
-    if(status){
+  if(internal->output_p){
+    internal->output_p=0;
+    internal->isStopping = true;
+
+    if(!internal->started && internal->validByteCount){
+      status = AudioOutputUnitStart(internal->outputAudioUnit);
+      adebug("Starting audio output unit\n");
+      if(status){
+        pthread_mutex_unlock(&mutex);
+        aerror("Failed to start audio output => %d\n",(int)status);
+        return 0;
+      }
+      internal->started = true;
+    }
+
+    // For some rare cases (using atexit in your program) Coreaudio tears
+    // down the HAL itself, so we do not need to do that here.
+    // We wouldn't get an error if we did, but AO would hang waiting for the 
+    // AU to flush the buffer
+    sizeof_running = sizeof(UInt32);
+    AudioUnitGetProperty(internal->outputAudioUnit, 
+                         kAudioDevicePropertyDeviceIsRunning,
+                         kAudioUnitScope_Input,
+                         0,
+                         &running, 
+                         &sizeof_running);
+
+    if (!running) {
       pthread_mutex_unlock(&mutex);
-      aerror("Failed to start audio output => %d\n",(int)status);
-      return 0;
+      return 1;
     }
-    internal->started = true;
-  }
 
-  // For some rare cases (using atexit in your program) Coreaudio tears
-  // down the HAL itself, so we do not need to do that here.
-  // We wouldn't get an error if we did, but AO would hang waiting for the 
-  // AU to flush the buffer
-  sizeof_running = sizeof(UInt32);
-  AudioUnitGetProperty(internal->outputAudioUnit, 
-		       kAudioDevicePropertyDeviceIsRunning,
-		       kAudioUnitScope_Input,
-		       0,
-		       &running, 
-		       &sizeof_running);
+    // Only stop if we ever got started
+    if (internal->started) {
 
-  if (!running) {
-    pthread_mutex_unlock(&mutex);
-    return 1;
-  }
+      // Wait for any pending data to get flushed
+      while (internal->validByteCount)
+        pthread_cond_wait(&cond, &mutex);
 
-  // Only stop if we ever got started
-  if (internal->started) {
-
-    // Wait for any pending data to get flushed
-    while (internal->validByteCount)
-      pthread_cond_wait(&cond, &mutex);
-
-    pthread_mutex_unlock(&mutex);
+      pthread_mutex_unlock(&mutex);
     
-    status = AudioOutputUnitStop(internal->outputAudioUnit);
-    if (status) {
-      awarn("AudioOutputUnitStop returned %d\n", (int)status);
-      return 0;
-    }
+      status = AudioOutputUnitStop(internal->outputAudioUnit);
+      if (status) {
+        awarn("AudioOutputUnitStop returned %d\n", (int)status);
+        return 0;
+      }
 
-    status = AudioUnitUninitialize(internal->outputAudioUnit);
-    if (status) {
-      awarn("AudioUnitUninitialize returned %d\n", (int)status);
-      return 0;
-    }
+      status = CloseComponent(internal->outputAudioUnit);
+      if (status) {
+        awarn("CloseComponent returned %d\n", (int)status);
+        return 0;
+      }
+    }else
+      pthread_mutex_unlock(&mutex);
   }else
     pthread_mutex_unlock(&mutex);
 
@@ -536,7 +555,9 @@ void ao_plugin_device_clear(ao_device *device)
 {
   ao_macosx_internal *internal = (ao_macosx_internal *) device->internal;
 
-  free(internal->buffer);
+  if(internal->buffer)
+    free(internal->buffer);
   free(internal);
+  device->internal=NULL;
 }
 
